package java.time;

import java.util.Collections;
import java.util.HashMap;
import java.io.IOException;
import java.io.DataOutput;
import java.io.InvalidObjectException;
import java.io.ObjectInputStream;
import java.time.zone.ZoneRulesException;
import java.time.zone.ZoneRules;
import java.time.temporal.TemporalQuery;
import java.time.temporal.UnsupportedTemporalTypeException;
import java.time.temporal.TemporalField;
import java.time.format.DateTimeFormatterBuilder;
import java.util.Locale;
import java.time.format.TextStyle;
import java.time.temporal.TemporalQueries;
import java.time.temporal.TemporalAccessor;
import java.util.Objects;
import java.time.zone.ZoneRulesProvider;
import java.util.Set;
import java.util.TimeZone;
import java.util.Map;
import java.io.Serializable;

public abstract class ZoneId implements Serializable
{
    public static final Map<String, String> SHORT_IDS;
    private static final long serialVersionUID = 8352817235686L;
    
    public static ZoneId systemDefault() {
        return TimeZone.getDefault().toZoneId();
    }
    
    public static Set<String> getAvailableZoneIds() {
        return ZoneRulesProvider.getAvailableZoneIds();
    }
    
    public static ZoneId of(String obj, Map<String, String> obj2) {
        Objects.requireNonNull(obj, "zoneId");
        Objects.requireNonNull(obj2, "aliasMap");
        String s = (String)obj2.get(obj);
        obj = ((s != null) ? s : obj);
        return of(obj);
    }
    
    public static ZoneId of(String s) {
        return of(s, true);
    }
    
    public static ZoneId ofOffset(String concat, ZoneOffset obj) {
        Objects.requireNonNull(concat, "prefix");
        Objects.requireNonNull(obj, "offset");
        if (concat.length() == 0) {
            return obj;
        }
        if (!concat.equals("GMT") && !concat.equals("UTC") && !concat.equals("UT")) {
            throw new IllegalArgumentException("prefix should be GMT, UTC or UT, is: " + concat);
        }
        if (obj.getTotalSeconds() != 0) {
            concat = concat.concat(obj.getId());
        }
        return new ZoneRegion(concat, obj.getRules());
    }
    
    static ZoneId of(String obj, boolean b) {
        Objects.requireNonNull(obj, "zoneId");
        if (obj.length() <= 1 || obj.startsWith("+") || obj.startsWith("-")) {
            return ZoneOffset.of(obj);
        }
        if (obj.startsWith("UTC") || obj.startsWith("GMT")) {
            return ofWithPrefix(obj, 3, b);
        }
        if (obj.startsWith("UT")) {
            return ofWithPrefix(obj, 2, b);
        }
        return ZoneRegion.ofId(obj, b);
    }
    
    private static ZoneId ofWithPrefix(String str, int n, boolean b) {
        String substring = str.substring(0, n);
        if (str.length() == n) {
            return ofOffset(substring, ZoneOffset.UTC);
        }
        if (str.charAt(n) != '+' && str.charAt(n) != '-') {
            return ZoneRegion.ofId(str, b);
        }
        try {
            ZoneOffset of = ZoneOffset.of(str.substring(n));
            if (of == ZoneOffset.UTC) {
                return ofOffset(substring, of);
            }
            return ofOffset(substring, of);
        }
        catch (DateTimeException ex) {
            throw new DateTimeException("Invalid ID for offset-based ZoneId: " + str, ex);
        }
    }
    
    public static ZoneId from(TemporalAccessor obj) {
        ZoneId zoneId = (ZoneId)obj.query(TemporalQueries.zone());
        if (zoneId == null) {
            throw new DateTimeException("Unable to obtain ZoneId from TemporalAccessor: " + obj + " of type " + obj.getClass().getName());
        }
        return zoneId;
    }
    
    ZoneId() {
        if (this.getClass() != ZoneOffset.class && this.getClass() != ZoneRegion.class) {
            throw new AssertionError("Invalid subclass");
        }
        return;
    }
    
    public abstract String getId();
    
    public String getDisplayName(TextStyle textStyle, Locale locale) {
        return new DateTimeFormatterBuilder().appendZoneText(textStyle).toFormatter(locale).format(this.toTemporal());
    }
    
    private TemporalAccessor toTemporal() {
        return new TemporalAccessor() {
            @Override
            public boolean isSupported(TemporalField temporalField) {
                return false;
            }
            
            @Override
            public long getLong(TemporalField obj) {
                throw new UnsupportedTemporalTypeException("Unsupported field: " + obj);
            }
            
            @Override
            public <R> R query(TemporalQuery<R> query) {
                if (query == TemporalQueries.zoneId()) {
                    return ZoneId.this;
                }
                return super.query(query);
            }
        };
    }
    
    public abstract ZoneRules getRules();
    
    public ZoneId normalized() {
        try {
            ZoneRules rules = this.getRules();
            if (rules.isFixedOffset()) {
                return rules.getOffset(Instant.EPOCH);
            }
        }
        catch (ZoneRulesException ex) {}
        return this;
    }
    
    @Override
    public boolean equals(Object o) {
        return this == o || (o instanceof ZoneId && this.getId().equals(((ZoneId)o).getId()));
    }
    
    @Override
    public int hashCode() {
        return this.getId().hashCode();
    }
    
    private void readObject(ObjectInputStream objectInputStream) throws InvalidObjectException {
        throw new InvalidObjectException("Deserialization via serialization delegate");
    }
    
    @Override
    public String toString() {
        return this.getId();
    }
    
    private Object writeReplace() {
        return new Ser((byte)7, this);
    }
    
    abstract void write(DataOutput p0) throws IOException;
    
    static {
        HashMap m = new HashMap(64);
        m.put("ACT", "Australia/Darwin");
        m.put("AET", "Australia/Sydney");
        m.put("AGT", "America/Argentina/Buenos_Aires");
        m.put("ART", "Africa/Cairo");
        m.put("AST", "America/Anchorage");
        m.put("BET", "America/Sao_Paulo");
        m.put("BST", "Asia/Dhaka");
        m.put("CAT", "Africa/Harare");
        m.put("CNT", "America/St_Johns");
        m.put("CST", "America/Chicago");
        m.put("CTT", "Asia/Shanghai");
        m.put("EAT", "Africa/Addis_Ababa");
        m.put("ECT", "Europe/Paris");
        m.put("IET", "America/Indiana/Indianapolis");
        m.put("IST", "Asia/Kolkata");
        m.put("JST", "Asia/Tokyo");
        m.put("MIT", "Pacific/Apia");
        m.put("NET", "Asia/Yerevan");
        m.put("NST", "Pacific/Auckland");
        m.put("PLT", "Asia/Karachi");
        m.put("PNT", "America/Phoenix");
        m.put("PRT", "America/Puerto_Rico");
        m.put("PST", "America/Los_Angeles");
        m.put("SST", "Pacific/Guadalcanal");
        m.put("VST", "Asia/Ho_Chi_Minh");
        m.put("EST", "-05:00");
        m.put("MST", "-07:00");
        m.put("HST", "-10:00");
        SHORT_IDS = Collections.unmodifiableMap(m);
    }
}
